<?php

	/**
	*	Search though all applications using the linking system, display all matching results
	*	up to RESULT_COUNT for each application.  If there are more results, and the application supports the view_list
	*	hook, a 'More...' link will be displayed to take the user to the full list of results for that app.
	*
	*	@author Nathan Gray
	*	@package stylite
	*	@copyright (c) 2003-9 by Nathan Gray <NathanGray@users.sourceforge.net>
	*	@license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
	*/

	class stylite_link_search_ui {

		const RESULT_COUNT = 10;

		public $public_functions = array(
			'search'	=>	true
		);

		protected static $template;

		public function __construct() {
			if(!is_object(self::$template)) {
				self::$template = new etemplate();
			}
		}

		public function search($content = array()) {
			$data = array();
			$apps = egw_link::app_list('query');

			// Skip these
			unset($apps['addressbook-email']);

			// Only include accounts if account storage is different from contact storage
			$addressbook = new addressbook_so();
			if($addressbook->account_repository == $addressbook->contact_repository)
			{
				unset($apps['home-accounts']);
			}
			if ($apps)
			{
				asort($apps);   // sort them alphabetic

				foreach($apps as $app => $app_name) {
					$app_result = array(
						'app'		=>	$app,
						'appname'	=>	lang($app),
						'icon'		=>	$app . '/navbar',
					);
					if(egw_link::$app_register[$app]['view_list']) {
						$app_result['list'] = egw::link('/index.php', array(
							'menuaction'	=>	egw_link::$app_register[$app]['view_list']
						));
					}

					// Sub-types
					if(is_array(egw_link::$app_register[$app]['types'])) {
						foreach(egw_link::$app_register[$app]['types'] as $key => $type) {
							$app_result['type'] = $key;
							$app_result['appname'] = lang($app) . ' - ' . lang($type['name']);
							$app_result['icon'] = $type['icon'] ? $type['icon'] : $app_result['icon'];
			
							$data['apps'][] = $app_result;
						}
					} else {
						$data['apps'][] = $app_result;
					}
				}
			}

			$data['query'] = $content['query'];

			// Make DIVs to hold results
			$data['apps'] = self::make_app_boxes($data['apps']);

			// xajax
			$GLOBALS['egw_info']['flags']['include_xajax'] = True;

			// JavaScript
			if(!is_object($GLOBALS['egw']->js)) {
				$GLOBALS['egw']->js =& CreateObject('phpgwapi.javascript');
			}
			$GLOBALS['egw']->js->validate_file('link_search', 'link_search', 'stylite');
			if(trim($data['query']) != '') {
				$GLOBALS['egw']->js->set_onload("search('" . $data['query'] . "');");
			}

			$GLOBALS['egw_info']['flags']['app_header'] = lang('Site search');
			self::$template = new etemplate('stylite.link_search.search');
			self::$template->exec('stylite.stylite_link_search_ui.search', $data, $select_box_options, $readonly, $preserve);
		}

		/**
		*	Make some DIVs to hold the results
		*/
		protected static function make_app_boxes($apps) {
			if(!is_array($apps)) {
				return array();
			}
			
			$result = array();
			foreach($apps as $app) {
				self::$template = new etemplate('stylite.link_search.app');
				$result[] = sprintf('<div id="%s" class="app_box" data-app="%s" data-type="%s" data-link="%s">%s</div>',
					$app['app'],
					$app['app'],
					$app['type'],
					$app['list'],
					self::$template->show($app)
				);
			}
			return implode('', $result);
		}

		public static function query($app, $pattern, &$options = array(), $etemplate_request_id = '') {
			$result = false;
			$results = array();
			$app_result = array();
			$options['start']	=	0;
			$options['num_rows']	=	self::RESULT_COUNT;

			try {
				// Make sure any extra output doesn't spoil the results
				ob_start();
				$results = egw_link::query($app, $pattern, $options);
				if(is_array($results) && count($results) > 0) {
					foreach($results as $id => $title) {
						$new_line = array(
							'app'	=>	$app,
							'id'	=>	$id,
						);
						if(is_array($title)) {
							$new_line['title'] = $title['label'];
							$new_line = array_merge($new_line, $title);
						} else {
							$new_line['title'] = $title;
						}
						$app_result[] = $new_line;
					}
					self::$template = new etemplate('stylite.link_search.app_results');
					if($etemplate_request_id) etemplate::$request = etemplate_request::read($etemplate_request_id);
					$result = self::$template->show($app_result);
				}

				// Send any extra output to the error log
				if(ob_get_length() > 0) {
					error_log("Extra output searching $app: " . ob_get_contents());
				}
				ob_end_clean();
			} catch (Exception $e) {
				error_log($e->getMessage());
			}
			return $result;
		}
	}
?>
