<?php
/**
 * Stylite: Mobil calendar
 *
 * @link http://www.stylite.de
 * @package stylite
 * @subpackage calendar
 * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @copyright (c) 2009/10 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @version $Id: class.stylite_mobil.inc.php 854 2010-01-04 22:35:31Z ralfbecker $
 */

/**
 * EPL app for mobile devices
 *
 * Shows only a calendar currently
 * 
 * @todo use screen size (self::$ua2width[self::$ua]) to adapt layout
 */
class stylite_mobil extends calendar_ui
{
	const APP_NAME = 'calendar';
	
	/**
	 * Minimum width for two colum layout
	 */
	const MIN_WIDTH_TWO_COLUMN_LAYOUT = 314;

	/**
	 * Minimum width for jscalendar
	 */
	const MIN_JSCAL_WIDTH = 167;

	/**
	 * Maximum width for jscalendar
	 */
	const MAX_JSCAL_WIDTH = 200;

	/**
	 * Methods callable via menuaction
	 *
	 * @var array
	 */
	public $public_functions = array(
		'index' => true,
		//'edit' => true,
	);
	
	/**
	 * User agent
	 * 
	 * @var string
	 */
	static public $user_agent;
	/**
	 * Screen width per user-agent
	 * 
	 * We do it per user agent, in case basic auth session get's shared by different devices
	 * 
	 * @var array
	 */
	static public $ua2width;
	/**
	 * Screen height per user-agent
	 * 
	 * @var array
	 */
	static public $ua2height;

	/**
	 * Constructor
	 *
	 */
	public function __construct()
	{
		parent::__construct(false,$_REQUEST);
		
		// get and store screen size per user-agent in the session
		if (!isset(self::$ua2width))
		{
			self::$ua2width =& egw_cache::getSession(__CLASS__,'ua2width',create_function('','return array();'));
			self::$ua2height =& egw_cache::getSession(__CLASS__,'ua2height',create_function('','return array();'));
		}
		self::$user_agent = $_SERVER['HTTP_USER_AGENT'];
		if (!isset(self::$ua2width[self::$user_agent]) || isset($_GET['width']))
		{
			self::getScreenSize();
		}
	}
	
	/**
	 * Get size of screen via javascript or from the url
	 */
	public function getScreenSize()
	{
		if (isset($_GET['width']))
		{
			self::$ua2width[self::$user_agent] = (int)$_GET['width'];
			self::$ua2height[self::$user_agent] = (int)$_GET['height'];
		}
		// we use a static size for iPhone viewport, as iPhone would report it's viewport not screensize below
		// and touchscreen requires far bigger fontsizes!
		elseif (strpos(self::$user_agent,'iPhone') !== false)
		{
			self::$ua2width[self::$user_agent] = 200;	// 320 regular, but we use 200px viewport
			self::$ua2height[self::$user_agent] = 300;	// 480 regular
		}
		else
		{
			echo "<html>\n<head>\n\t<title>Get Screen Size</title>\n";
			// add a permanent redirect in case browser has no javascript support with default width/height
			// switched off as it seems to make problems with blackberry AND app does not work without js anyway
			//echo "\t<meta http-equiv=refresh content='1; URL=".egw::link('/stylite/mobil.php?width=320&height=240')."'</meta>\n";
			echo "</head>\n<body>
<script>
function getScreenSize()
{
	var width = window.innerWidth ? window.innerWidth : document.body.clientWidth;
	var height = window.innerHeight ? window.innerHeight : document.body.clientHeight;
	//alert('Width='+width+' Height='+height);
	document.location.href = document.location.href+'?width='+width+'&height='+height;
}
getScreenSize();
</script>
<noscript>
<h1>Javascript required!</h1>
</noscript>\n</body>\n</html>\n";
			common::egw_exit();
		}
	}

	/**
	 * Display calendar view
	 *
	 */
	public function index()
	{
		//echo "<p>this->view = '$this->view', this->date = '$this->date', this->owner = '$this->owner'</p>\n";
		
		$screen_width = self::$ua2width[self::$user_agent];
		$screen_height = self::$ua2height[self::$user_agent];
		// if screen is landscape AND width >= 320px --> use 2 columns
		if ($screen_width > $screen_height && $screen_width >= self::MIN_WIDTH_TWO_COLUMN_LAYOUT)
		{
			$cal_width = $screen_width / 2;
			if ($cal_width < self::MIN_JSCAL_WIDTH) $cal_width = self::MIN_JSCAL_WIDTH;	// abs. minimum for jscalendar
			if ($cal_width > self::MAX_JSCAL_WIDTH) $cal_width = self::MAX_JSCAL_WIDTH;	// more makes little sense either
			$event_width = $screen_width - $cal_width;
			$two_column_layout = true;
		}
		else	// use single column with full width
		{
			$cal_width = $event_width = $screen_width;
		}
		//echo "<p>screen=$screen_width*$screen_height: cal=$cal_width, events=$event_width</p>\n";

		if (!in_array($this->view,array('day','listview')))
		{
			$this->view = 'day';	// our default
		}

		// Calendarselection: User or Group
		if(count($this->bo->grants) > 0 && $this->accountsel->account_selection != 'none')
		{
			$grants = array();
			foreach($this->bo->list_cals() as $grant)
			{
				$grants[] = $grant['grantor'];
			}
			// exclude non-accounts from the account-selection
			$accounts = array();
			foreach(explode(',',$this->owner) as $owner)
			{
				if (is_numeric($owner)) $accounts[] = $owner;
			}
			if (!$accounts) $grants[''] = lang('None');
			$calendar = "
<script type=\"text/javascript\">
function load_cal(url,id) {
	var owner='';
	selectBox = document.getElementById(id);
	for(i=0; i < selectBox.length; ++i) {
		if (selectBox.options[i].selected) {
			owner += (owner ? ',' : '') + selectBox.options[i].value;
		}
	}
	if (owner) {
		location=url+'&owner='+owner;
	}
}
</script>
".
			html::div($this->accountsel->selection('owner','uical_select_owner',$accounts,'calendar+',count($accounts) > 1 ? 4 : 1,False,
				' style="width: '.(count($accounts) > 1 && in_array($this->common_prefs['account_selection'],array('selectbox','groupmembers')) ? 
					'100%' : ($cal_width-20).'px').';"'.
				' title="'.lang('select a %1',lang('user')).'" onchange="load_cal(\''.
				egw::link('/stylite/mobil.php',array(
					'view' => $this->view,
					'date' => $this->date,
				)).'\',\'uical_select_owner\');"','',$grants),' id="mobilOwnerSelection"');
		}

		// Search (only for list view or two-column-layout)
		if ($this->view == 'listview' || $two_column_layout)
		{
			$blur = addslashes(html::htmlspecialchars(lang('Search').' ...'));
			$value = !empty($_REQUEST['keywords']) ? $_REQUEST['keywords'] : $blur;
			$search = html::form(html::input('keywords',$value,'text',' id="mobilSearch"'.
				' onFocus="if(this.value==\''.$blur.'\') this.value=\'\';"'.
				' onBlur="if(this.value==\'\') this.value=\''.$blur.'\';" title="'.htmlspecialchars(lang('Search')).'"'),
				'','/stylite/mobil.php',array('view' => 'listview'),'',' id="mobilSearchForm"','GET');
		}

		// Minicalendar for dayview or two column layout
		if ($this->view == 'day' || $two_column_layout)
		{
			$link = array();
			foreach(array(
				'day'   => 'day',
				'week'  => 'listview',
				'month' => 'listview') as $name => $view)
			{
				$link[$name] = egw::link('/stylite/mobil.php',array('view' => $view));
			}
			$jscalendar = $this->jscal->flat($link['day'],$this->date,
				$link['week'],lang('show this week')/*,$link['month'],lang('show this month')*/);
		}

		// views with icons
		$views = array(
			'prev'=> array('icon'=>'previous_kde', 'text' => 'Previous'),
			'add' => array('icon'=>'new', 'text'=>'add'),
			'day' => array('icon'=>'today', 'text'=>'Today','view' => 'day','date' => egw_time::to('now','Ymd')),
			'list'=> array('icon'=>'list', 'text'=>'Listview','view' => 'listview'),
			'next'=> array('icon'=>'next_kde', 'text' => 'Next'),
		);

		// query events
		$end = new egw_time($this->date);
		$prev = clone $next = clone $end;
		if (empty($_REQUEST['keywords']))	// no search
		{
			$params = array(
				'start'   => $this->date,
				'users'   => explode(',',$this->owner),
				'enum_recuring' => true,
				'order'   => 'cal_start',
				'daywise' => true,
				'filter' => 'default',
			);
			if ($this->view == 'day')
			{
				$end->modify('+2day -1second');
				$prev->modify('-1day');
				$next->modify('+1day');
			}
			else
			{
				$end->modify('+7day -1second');
				$prev->modify('-7day');
				$next->modify('+7day');
			}
			$params['end'] = $end;
			// dates for prev and next icons
			$views['prev']['date'] = $prev->format('Ymd');
			$views['next']['date'] = $next->format('Ymd');
		}
		else	// search
		{
			// we currently search form today displaying todays events first followed by future ones
			$params = array(
				'start'   => $this->bo->now_su,
				'users'   => explode(',',$this->owner),
				'enum_recuring' => true,	// query does NOT work without enum_recuring!!!
				//'order'   => 'cal_start DESC',
				'query' => $_REQUEST['keywords'],
				'offset' => (int)$_REQUEST['offset'],
				'filter' => 'all',
			);
			// search uses icons to scroll, rather then change date
			$views['prev']['offset'] = max(0,$_REQUEST['offset']-$this->common_prefs['maxmatchs']);
			$views['next']['offset'] = $_REQUEST['offset']+$this->common_prefs['maxmatchs'];
			$views['prev']['keywords'] = $views['next']['keywords'] = $_REQUEST['keywords'];
		}
		//_debug_array($params);
		
		// get icons
		$icons = '<table id="mobilIcons"><tr>'."\n";
		foreach($views as $view => $data)
		{
			$icon = array_shift($data);
			$title = array_shift($data);

			$icon = html::image('calendar',$icon,lang($title),"class=sideboxstar");  //to avoid jscadender from not displaying with pngfix
			$link = $view == 'add' ? $this->add_link($icon,null,null,0,array('template' => 'stylite.mobil.edit')) : 
				html::a_href($icon,'/stylite/mobil.php',$data);

			$icons .= '<td align="center">'.$link."</td>\n";
		}
		$icons .= "</tr></table>\n";

		// calendar list
		$last_day = null;
		foreach($this->bo->search($params) as $day => $day_events)
		{
			//var_dump($day); _debug_array($day_events);
			// make a non-daywise view (used for search) look like a daywise one
			if (!$params['daywise'])
			{
				$day = egw_time::to($day_events['start'],'Ymd');
				$day_events = array($day_events);
			}
			// add a day header
			if ($last_day != $day)
			{
				// header with date and add icon for listview or two-column-layout
				if ($this->view != 'day' || $two_column_layout)
				{
					$dayo = new egw_time((string)$day);
					$add_link = $this->add_link(html::image('calendar','new'),$day,null,null,array(
						'template' => 'stylite.mobil.edit',
					));
					$events .= html::div(
						lang($dayo->format('l')).' '.$dayo->format(true).
						'<span class="mobilDayHeaderAdd">'.$add_link.'</span>'
						,' class="mobilDayHeader"')."\n";
				}
				// only a header with "tomorrow" for day-view in one-column-layout
				elseif ($day != $this->date)
				{
					$events .= html::div(lang('tomorrow'),' class="mobilDayHeader"')."\n";
				}
				$last_day = $day;
			}
			foreach($day_events as $event)
			{
				$start_today = egw_time::to($event['start'],'Ymd') == $day;
				$end_today = egw_time::to($event['end'],'Ymd') == $day;
				$wholeday = !$start_today && !$end_today ||
					egw_time::to($event['start'],'H:i') === '00:00' &&
						egw_time::to($event['end'],'H:i') === '23:59';
				if (!$start_today && $end_today) $event['start'] = (string)$day;	// start at 0h
				$events .= html::div(html::a_href(
					($wholeday ? html::italic(lang('whole day')) : egw_time::to($event['start'],false)).' '.
					// make title of private events italic, to diver from title "Private"
					(!$this->bo->check_perms(EGW_ACL_READ,$event) ? html::italic(lang('Private')) : $event['title']).
					($event['recur_type'] ? ' '.html::image('calendar','recur',lang('recurring event'),' style="width: 12px;"') : ''),
					'/index.php',array(
						'menuaction' => 'calendar.calendar_uiforms.edit',
						'template' => 'stylite.mobil.edit',
						'cal_id' => $event['id'],
					),' target="_blank" onclick="'.$this->popup('this.href','this.target').'; return false;"'),' class="mobilEvent"');
			}
		}
		// include our own css
		$css_file = '/stylite/templates/default/mobil.css';
		$GLOBALS['egw_info']['flags']['css'] .= "\n\t\t</style>\n\t\t".'<link href="'.$GLOBALS['egw_info']['server']['webserver_url'].
			$css_file.'?'.filemtime(EGW_SERVER_ROOT.$css_file).'" type="text/css" rel="StyleSheet" />'."\n".
			// special tag for iPhone to use a viewport of 200px width
			"\t\t".'<meta name="viewport" content="width='.$screen_width.'; user-scalable=false" />'."\n\t\t<style>\n";;
			
		$date = new egw_time($this->date);
		$GLOBALS['egw_info']['flags']['app_header'] = lang('Calendar').' '.
			(strpos($this->owner,',') === false ? common::grab_owner_name($this->owner).' ' : '').
			lang($date->format('l')).' '.$date->format(true);
		unset($GLOBALS['egw_info']['flags']['noheader']);

		$GLOBALS['egw']->framework->render(html::div(
			html::div(
				$icons."\n".
				$jscalendar."\n".
				$calendar."\n".
				$search."\n"//.
			,'id="mobilCalendar" style="width: '.$cal_width.'px;"','')."\n".
			html::div($events,'id="mobilEvents" style="width: '.$event_width.'px;"')
		,' style="width: '.$screen_width.'px;"'));
	}

	/**
	 * Return javascript to open a popup window: window.open(...)
	 *
	 * Reimplemented to change width/height to screensize (or 200x300 for iPhone)
	 *
	 * @param string $link link or this.href
	 * @param string $target='_blank' name of target or this.target
	 * @param int $width=750 width of the window
	 * @param int $height=400 height of the window
	 * @return string javascript (using single quotes)
	 */
	function popup($link,$target='_blank',$width=200,$height=400,$Link_confirm_abort='',$Link_confirm_text='')
 	{
 		$width = self::$ua2width[self::$user_agent];
 		$height = self::$ua2height[self::$user_agent];
 
 		return parent::popup($link,$target,$width,$height,$Link_confirm_abort,$Link_confirm_text);
	}
}

