<?php
/**
 * Stylite - Projectmanager Gantt chart direct datasource update
 *
 * @link www.stylite.de
 * @author Nathan Gray
 * @copyright (c) 2014 Nathan Gray
 * @package stylite
 * @version $Id$
 */

/**
 * Direct datasoure updating from Gantt chart
 *
 * Normally the gantt chart just updates the project element.  This
 * class directly updates the associated datasource (infolog, etc.)
 */
class stylite_projectmanager_gantt
{

	private static $field_map = array(
		'calendar' => array(
			'pe_planned_start' => 'start',
			'pe_planned_end'  => 'end',
			'pe_real_start'   => 'start',
			'pe_real_end'     => 'end'
		),
		'infolog' => array(
			'pe_completion'   => 'info_percent',
			'pe_planned_start'=> 'info_startdate',
			'pe_planned_end'  => 'info_enddate',
			'pe_real_start'   => 'info_startdate',
			'pe_real_end'     => 'info_enddate',
		),
		'timesheet' => array(
			// do NOT change anything in timesheet, as it is used to document times for accounting
		),
		'tracker' => array(
			'pe_completion'   => 'tr_completion',
			'pe_planned_start'=> 'tr_startdate',
			'pe_planned_end'  => 'tr_duedate',
			'pe_real_start'   => 'tr_startdate',
			'pe_real_end'     => 'tr_duedate'
		)
	);

	/**
	 * Add extra stuff for set confirmation dialog
	 *
	 * @param type $data
	 * @param type $sel_options
	 * @param type $readonlys
	 */
	public static function chart(&$template, &$data, &$sel_options, &$readonlys)
	{
		$template->setElementAttribute('gantt',	'onchange', 'return app.classes.stylite_gantt.change(ev, widget)');
		
		egw_framework::validate_file('','gantt','stylite');
	}

	/**
	 * User changed start date, duration or completion from gantt chart.
	 * Update datasource.
	 *
	 * @param Array $values
	 * @param string $mode Type of update.  One of ("resize", "progress", "move")
	 * @param Array $params Other values from the form
	 */
	public static function ajax_update($values, $mode, $params)
	{
		// We let projectmanager handle projects, links, milestones and
		// values that the data source doesn't
		if(!$values['pe_id'] || !$values['pe_app'])
		{
			return false;
		}

		// Find and read the entry with appropriate PM access
		$bo = new projectmanager_elements_bo();
		$ds = $bo->datasource($values['pe_app']);
		$entry = $ds->read($values['pe_app_id']);

		if(!$entry)
		{
			// No entry or no access
			return false;
		}

		// Read actual entry
		$classname = $values['pe_app'] == 'calendar' ? 'calendar_boupdate' : $values['pe_app'] . '_bo';
		if(class_exists($classname))
		{
			$entry_bo = new $classname();
			$entry = $entry_bo->read($values['pe_app_id']);
		}

		$keys = array();
		$end_key = 'pe_' . ($params['planned_times'] ? 'planned' : 'real') . '_end';
		if($mode == 'progress')
		{
			$keys['pe_completion'] = (int)($values['progress'] * 100).'%';
		}
		if(array_key_exists('start_date', $values))
		{
			$keys['pe_' . ($params['planned_times'] ? 'planned' : 'real') . '_start'] = egw_time::to($values['start_date'],'ts');
		}
		// Only set end date if it has been resized, or it already had an end date
		if(($mode == 'resize' || $entry && $entry[self::$field_map[$values['pe_app']][$end_key]]) && array_key_exists('end_date', $values))
		{
			$keys[$end_key] = egw_time::to($values['end_date'],'ts');
		}
		if($keys)
		{

			// To update project entry
			$bo->read(array('pe_id' => (int)$values['pe_id']));
			$project_update = 0;

			// Map PM values onto application
			$valid_update = false;
			foreach($keys as $name => $value)
			{
				if(self::$field_map[$values['pe_app']][$name])
				{
					//error_log("Updating {$name} from {$entry[self::$field_map[$values['pe_app']][$name]]} to $value");
					// Update entry
					$valid_update = true;
					$entry[self::$field_map[$values['pe_app']][$name]] = $value;

					// Update project element
					$bo->data[$name] = $value;
					$bo->data['pe_overwrite'] &= ~$ds->name2id[$name];
					$update_necessary |= $id;
				}
			}
			// Clear overridden end date if it's not being set
			if($valid_update && !$keys[$end_key])
			{
				$bo->data[$end_key] = '';
				$bo->data['pe_overwrite'] &= ~$ds->name2id[$end_key];
			}

			// Update application entry
			if($valid_update)
			{
				if(method_exists($entry_bo, 'write'))
				{
					$entry_bo->write($entry);
				}
				elseif (method_exists($entry_bo, 'save'))
				{
					$entry_bo->save($entry);
				}
				// Update project element
				$bo->save(null, null,$update_necessary);
				return $valid_update;
			}
		}

		// Not an update we can do anything with, let pm handle it
		return false;
	}

}