<?php

/**
 * Base class for charts
 *
 * @package stylite
 * @author Nathan Gray
 * @copyright 2011 Nathan Gray
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @version $Id: class.stylite_report.inc.php 2241 2012-07-05 15:55:58Z nathangray $
 */

$GLOBALS['egw_info']['flags']['currentapp'] = 'stylite';


/**
 * Base class for report type charts showing breakdown by category, etc.
 */

abstract class stylite_report
{

	public $public_functions = array(
		'index'	=> true,
	);

	/**
	 * Time slices
	 * Controls granularity of x axis, or pie charts for NOW
	 */
	const NOW = 'NOW';
	const YEAR = 'YEAR';
	const MONTH = 'MONTH';
	const WEEK = 'WEEK';
	const DAY = 'DAY';

	/**
	 * Time frames
	 * Relative times for start / end date
	 */
	const THIS_WEEK = 7;
	const LAST_WEEK = 8;
	const THIS_MONTH = 1;
	const LAST_3_MONTHS = 9;
	const LAST_MONTH = 2;
	const THIS_QUARTER = 3;
	const LAST_QUARTER = 4;
	const THIS_YEAR = 5;
	const LAST_YEAR = 6;
	const TWO_YEARS_AGO = 10;
	const THREE_YEARS_AGO = 11;

	/**
	 * Time frames, with human value.
	 * For presenting a list for the user to choose from.
	 */
	public static $time_frame = array(
		self::NOW		=> 'Current',
		self::THIS_WEEK		=> 'This week',
		self::LAST_WEEK		=> 'Last week',
		self::THIS_MONTH	=> 'This month',
		self::LAST_MONTH	=> 'Last month',
		self::LAST_3_MONTHS	=> 'Last 3 months',
		self::THIS_QUARTER	=> 'This quarter',
		self::LAST_QUARTER	=> 'Last quarter',
		self::THIS_YEAR		=> 'This year',
		self::LAST_YEAR		=> 'Last year',
		self::TWO_YEARS_AGO	=> '2 years ago',
		self::THREE_YEARS_AGO	=> '3 years ago',
	);

	/**
	 * Database date formats, for getting the database to group by date
	 */
	protected static $time_slice_query = array(
		self::YEAR	=> '%Y',
		self::MONTH	=> '%Y-%m',
		self::WEEK	=> '%Y-%u',
		self::DAY	=> '%Y-%m-%d',
	);

	/**
	 * PHP data formats, matching database formats above,
	 * for PHP to interpolate missing days.
	 */
	protected static $php_date_format = array(
		self::YEAR	=> 'Y',
		self::MONTH	=> 'Y-m',
		self::WEEK	=> 'Y-W',
		self::DAY	=> 'Y-m-d',
	);

	// Selectbox options for filters
	protected $select_options = array();

	// Group By options
	protected $group_by_options = array('cat_id' => 'category');

	// Allow for custom colors
	protected $current_colors = array();

	// Field to history for reading history log
	protected static $field2history = array();

	// Field to label for nicer labels & exports
	protected static $field2label = array();

	// Table to get current data - extending class should set
	protected $current_table = null;

	// Field to use to filter types (for customfields)
	protected $type_filter = null;

	// Instance for singleton
	public static $instance = null;

	/**
	 * Create a new report
	 *
	 * Everything is static, but framework needs a constructor for index()
	 */
	public function __construct()
	{
	}

	/**
	 * Extending classes need to implement this so everything stays as
	 * static as possible.
	 */
	public static abstract function instance();

	public static function static_init() {
	}

	/**
	 * Display user interface for choosing and editing reports
	 *
	 * @param content Array values returned from eTemplate
	 */
	public static function index($content = array())
	{
		$tmpl = new etemplate('stylite.report');

		$app = $GLOBALS['egw_info']['flags']['currentapp'] ? $GLOBALS['egw_info']['flags']['currentapp'] : 'stylite';
		$name = $content['new_name'] ? $content['new_name'] : $content['report_name'];
		unset($content['new_name']);

		// Remember old values so we can tell if they changed
		$preserve['old_report'] = $name;
		$preserve['old_time_frame'] = $content['parameters']['time_frame'];

		// Make sure there's a group by
		if($content['parameters']['time_frame'] !== self::NOW && !$content['parameters']['group_by']) {
			$content['parameters']['group_by'] = 'cat_id';
		}

		// Store report settings
		if($name && ($content['save'] || $content['update'])) {
			self::save_report($name, $app, $content['parameters']);
		}

		// Update view of graph if the time frame changed (graph type may change too)
		if($content['update'] || $content['parameters']['time_frame'] != $content['old_time_frame']) {
			// Store it in the session for later
			unset($name);
			egw_session::appsession('report', $app, $content['parameters']);
		} elseif($content['report_name'] && $content['report_name'] != $content['old_report']) {
			// Load selected report
			$content = self::load_report($content['report_name'], $app);
			$content['report_name'] = $name;
		} elseif($content['export']) {
			$content['parameters']['app'] = $app;
			self::export($content['parameters']);
		} elseif($content['delete'] && $content['report_name']) {
			self::delete_report($content['report_name'], $app);
			$content = array();
		}

		$data = $content;

		$static_class = call_user_func(array($app ? $app.'_report' : 'stylite_report', 'instance'),array());

		if(!$data['parameters']['time_frame']) $data['parameters']['time_frame'] = self::NOW;
		$sel_options = array(
			'time_frame' => self::$time_frame,
			'group_by' => $static_class->group_by_options,
			'report_name' => self::report_list($app),
		) + $static_class->select_options;
		$readonly = array();

		// Look for some custom fields that could work
		if(self::get_custom_fields($app, $data, $sel_options, $static_class->type_field ? $data[$static_class->type_field] : null)) {
			$tmpl->disable_cells('cf');
		}

		// Disable exporting?
		$config = config::read('phpgwapi');
		$limit_exception = count(@array_intersect(array($GLOBALS['egw_info']['user']['account_id']) + $GLOBALS['egw']->accounts->memberships($GLOBALS['egw_info']['user']['account_id'],true), unserialize($GLOBALS['egw_info']['server']['export_limit_excepted']))) > 0;
		if($config['export_limit'] == 'no' && !($GLOBALS['egw_info']['user']['apps']['admin'] || $limit_exception)) {
			$tmpl->disable_cells('export');
		}
		$data['parameters']['params_allowed'] = true;

		// Allow extending app a chance
		if($app != 'stylite') {
			if(!is_array($content['parameters'])) $content['parameters'] = array();
			if(!is_array($data['parameters'])) $data['parameters'] = array();

			$class = "{$app}_report";
			call_user_func_array(array($class, 'get_parameters'), array(&$content['parameters'], &$data['parameters'], &$sel_options, &$readonly));
		}
		$data['parameters']['no_group_by'] = $data['parameters']['time_frame'] == self::NOW;
		$data['report'] = html::image('stylite', array('menuaction' => 'stylite.'.$app . '_report.image','app' => $app, 'report' => $name), $name);
		$data['parameters']['report_parameters'] = $app !== 'stylite' ? "stylite.$app.report" : ''; // Don't use stylite, causes a loop

		return $tmpl->exec("stylite.{$app}_report.index", $data, $sel_options, $readonly, $preserve);
	}

	/**
	 * Look for some custom fields for filtering
	 *
	 * Allows additional filtering by selectbox or radio custom fields.
	 * Extending class needs to implement actual filtering.
	 *
	 * @param app Appname
	 * @param data Data array - cf sub-array will be populated
	 * @param sel_options array - Select options for valid custom fields will be populated
	 *
	 * @return boolean true if no custom fields, false if some found
	 */
	protected static function get_custom_fields($app, &$data, &$sel_options,$type_filter=null)
	{
		$custom = config::get_customfields($app,false,$type_filter);
		$no_custom_fields = true;
		foreach($custom as $c_name => $cf) {
			if($cf['type'] == 'select' || $cf['type'] == 'radio') {
				$no_custom_fields = false;
				$data['parameters']['cf'][] = array(
					'label'	=> $cf['label'],
					'name'	=> $c_name
				);
				$sel_options[$c_name] = $cf['values'];
				$sel_options[$c_name][''] = lang('None');
				$sel_options['group_by'][$c_name] = $cf['label'];
			}
		}
		return $no_custom_fields;
	}
	
	/**
	 * Get a list of saved reports
	 *
	 * @param app Optional app name, defaults to current app
	 *
	 * @return array of parameters
	 */
	public static function report_list($app = null) {
		$app = $app ? $app : $GLOBALS['egw_info']['flags']['currentapp'];
		$config = config::read($app);
		return $config['reports'] ? array_combine(array_keys($config['reports']), array_keys($config['reports'])) : array();
	}

	/**
	 * Load saved report parameters
	 *
	 * @param name Saved report name
	 * @param app Optional app name, defaults to current app
	 *
	 * @return array of parameters
	 */
	public static function load_report($name, $app = null) {
		$app = $app ? $app : $GLOBALS['egw_info']['flags']['currentapp'];
		$config = config::read($app);
		return $config['reports'][$name] ? $config['reports'][$name] : array();
	}

	/**
	 * Save report parameters
	 *
	 * @param name Report name to save
	 * @param appname Application
	 * @param parameters Array of report parameters
	 */
	public static function save_report($name, $app, $parameters) {
		$config = config::read($app);
		$reports = $config['reports'];
		$reports[$name] = $parameters;
		config::save_value('reports', $reports, $app);
	}

	/**
	 * Delete saved report
	 *
	 * @param name Report name to delete
	 * @param appname Application
	 */
	public static function delete_report($name, $app) {
		$config = config::read($app);
		$reports = $config['reports'];
		unset($reports[$name]);
		config::save_value('reports', $reports, $app);
	}

	/**
	 * Process parameters interactively for report
	 *
	 * Extending application class should implement this to fill application specific values.
	 *
	 * @param content Incoming data from form
	 * @param data Output data for etemplate
	 * @param sel_options Selectbox options
	 * @param readonlys
	 */
	public static abstract function get_parameters(Array &$content = array(), Array &$data = array(), Array &$sel_options = array(), Array &$readonlys = array());

	/**
	 * Get data for a historical bar graph showing changes over time
	 *
	 * The strategy is to start with the known current values, then adjust these values
	 * based on the changes recorded in the history log.  Note that this is different from
	 * a report that shows the age of current entries.
	 * This is only a reference/test implementation for simple apps.  For applications with record-level ACL,
	 * this function may expose restricted data.
	 *
	 * @param time_slice How finely to slice the time - monthly, weekly, etc.
	 * @param start Earliest date in the graph
	 * @param end Latest date in the graph
	 * @param filters List of field filters
	 * @param group_by How to aggregate the data
	 *
	 * @return array of data for further use
	 */
	public static function get_history(Array $parameters) {

		// Needed until static:: becomes available
		$static_class = call_user_func(array($parameters['app'] . '_report', 'instance'),array());

		// Historical data (changes over time) preserved for posterity
		$data = array();
		$last_date = array();
		$filter = array();

		// Get current values
		$current = array(
			'table'	=> $static_class->current_table,
			'cols'	=> array(
				$GLOBALS['egw']->db->date_format('now()', self::$time_slice_query[$parameters['time_slice']]) . ' AS date',
				$parameters['group_by'] .' AS new',
				'"" AS old',
				'count(*) AS count'
			),
			'where'	=> $filter,
			'append'=> 'GROUP BY date, new, old',
			'app'	=> $parameters['app']
		);
		// Changes over time
		$history = array(
			'table'	=> historylog::TABLE,
			'cols'	=> array(
				$GLOBALS['egw']->db->date_format('history_timestamp', self::$time_slice_query[$parameters['time_slice']]) . ' AS date',
				'history_new_value AS new',
				'history_old_value AS old',
				'count(*) AS count'
			),
			'where' => array(
				'history_status' => $static_class->field2history[$parameters['group_by']],
				'history_appname' => $parameters['app'],
				'history_timestamp >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($parameters['start'])),
				'history_timestamp <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($parameters['end'])),
			),
			'append'=> 'GROUP BY date, new, old',
		);
		$result = $GLOBALS['egw']->db->union(array($current, $history), __LINE__, __FILE__, 'ORDER BY date DESC', 0, -1);
		foreach($result as $row) {
			// If there were changes today, move them into yesterday
			if($last_date[$row['new']] && $row['date'] == date(self::$php_date_format[$parameters['time_slice']])) {
				$row_date = date(self::$php_date_format[$parameters['time_slice']], strtotime('-1 ' . $parameters['time_slice']));
			} else {
				$row_date = $row['date'];
			}

			// Adjust counts based on changes in history
			if(!$last_date[$row['new']]) {
				$data[$row['new']][$row_date] = $row['count'];
			} else {
				$data[$row['new']][$row_date] = ($data[$row['new']][$last_date[$row['new']]]) - $row['count'];
			}
			if($row['old']) {
				$data[$row['old']][$row_date] = $data[$row['old']][$last_date[$row['old']]] + $row['count'];
				$last_date[$row['old']] = $row_date;
			}
			$last_date[$row['new']] = $row_date;
		}

		// Interpolate data for dates where there were no changes - assume the same as before there was a change
		$date = $parameters['end'];
		$last_series_date = null;
		while($date >= $parameters['start']) {
			$series_date = date(self::$php_date_format[$parameters['time_slice']], $date);
			foreach($data as $series => $values) {
				if(!$last_series_date && !$values[$series_date] && count($values)) {
					// Dates after current (user selected a full month on the 20th)
					// Shouldn't happen, but use the latest data and carry it forward
					$data[$series][$series_date] = current($values);
				} elseif(count($values) && !isset($values[$series_date])) {
					// No changes in this time period, copy newer value back
					$data[$series][$series_date] = $data[$series][$last_series_date];
				} elseif(!isset($values[$series_date])) {
					// No changes, no newer value = set to 0
					$data[$series][$series_date] = 0;
				}
			}
			$last_series_date = $series_date;
			$date = strtotime('-1 ' . $parameters['time_slice'], $date);
		}

		// Sort data so most recent date is at the end
		$pop_current = ($parameters['end'] < mktime(0,0,0,date('m'),date('d'),date('Y')));
		foreach($data as $series => &$values) {
			// If looking at the past, remove the current
			if ($pop_current) {
				unset($data[$series][date(self::$php_date_format[$parameters['time_slice']])]);
			}

			ksort($values);
		}

/*  Helpful debug, when you're working with the data
echo egw_time::to($parameters['start']) . ' - ' . egw_time::to($parameters['end']);
_debug_array($data);
echo '<table>';
		foreach($data as $series => $values) {
			echo "<tr><td>$series</td>";
			foreach($values as $date => $count) {
				echo "<td>$date</td><td>$count</td>";
			}
			echo "</tr>";
		}
echo '</table>';
	die();
//*/
		return $data;
	}

	/**
	 * Get aged report showing summary by opened date
	 *
	 * This gets current entries, for display by how old they are.
	 *
	 * @param time_slice How finely to slice the time - monthly, weekly, etc.
	 * @param start Earliest date in the graph
	 * @param end Latest date in the graph
	 * @param filters List of field filters
	 * @param group_by How to aggregate the data
	 *
	 * @return array of data for further use
	 */
	public static function get_aged(Array $parameters) {
		// Needed until static:: becomes available
		$static_class = call_user_func(array($parameters['app'] . '_report', 'instance'),array());

		$filter = array(
			$static_class->date_field . ' >= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($parameters['start'])),
			$static_class->date_field . ' <= ' . $GLOBALS['egw']->db->quote($GLOBALS['egw']->db->to_timestamp($parameters['end'])),
		);

		if(!$parameters['group_by'])
		{
			reset($static_class->group_by_options);
			$parameters['group_by'] = key($static_class->group_by_options);
		}
		
		// Add filters
		foreach($parameters as $field => $value) {
			if($value && in_array($field, array_keys($static_class->field2label))) {
				$filter[] = $GLOBALS['egw']->db->name_quote($field) . (is_array($value) ? ' IN ('.implode(',',$value).')':' = ' . $GLOBALS['egw']->db->quote($value));
			}
		}
		$result = $GLOBALS['egw']->db->select(
			$static_class->current_table,
			array(
				$GLOBALS['egw']->db->date_format($static_class->date_field, self::$time_slice_query[$parameters['time_slice']]) . ' AS date',
				$parameters['group_by'] .' AS value',
				'count(*) AS count'
			),
			$filter,
			__LINE__, __FILE__, false,
			'GROUP BY date, value',
			$parameters['app'], -1,
			$parameters['join']
		);

		$data = array();
		foreach($result as $row) {
			$data[$row['value']][$row['date']] = (int)$row['count'];
		}

		// Interpolate data for dates where there were no entries opened - value is 0
		$date = $parameters['end'];
		while($date >= $parameters['start']) {
			$series_date = date(self::$php_date_format[$parameters['time_slice']], $date);
			foreach($data as $series => $values) {
				if(!$values[$series_date]) {
					$data[$series][$series_date] = 0;
				}
			}
			$date = strtotime('-1 ' . $parameters['time_slice'], $date);
		}

		// Sort data so most recent date is at the end
		$pop_current = ($parameters['end'] < mktime(0,0,0,date('m'),date('d'),date('Y')));
		foreach($data as $series => &$values) {
			// If looking at the past, remove the current
			if ($pop_current) {
				unset($data[$series][date(self::$php_date_format[$parameters['time_slice']])]);
			}
			ksort($values);

		}

/*
echo egw_time::to($parameters['start']) . ' - ' . egw_time::to($parameters['end']);
_debug_array($data);
echo '<table>';
		foreach($data as $series => $values) {
			echo "<tr><td>$series</td>";
			foreach($values as $date => $count) {
				echo "<td>$date</td><td>$count</td>";
			}
			echo "</tr>";
		}
echo '</table>';
	die();
//*/

		return $data;
	}

	/**
	 * Get data for current settings
	 *
	 * Graph is made up of multiple pie charts showing the current breakdown.
	 * The data array should have one Name => pie_data entry for each pie desired.
	 * pie_data should have a Legend => Count entry for each slice.
	 *
	 * @param parameters Filters to filter data
	 * @return Array
	 */
	public abstract static function get_current(Array $parameters, $translate_field = true);

	/**
	 * Export to CSV
	 *
	 * Exports the data used for building the graphs, not the underlying source data.
	 *
	 * @parameters Array of filters, passed off to get_current() or get_aged(), same as for graphs.
	 */
	public static function export($parameters = array()) {
		if(!$parameters) {
			$name = trim($_GET['report']);
			$app = $_GET['app'];
			if($name) {
				$parameters = self::load_report($name);
			} elseif($app) {
				$name = lang($app);
				$parameters = egw_session::appsession('report', $app);
			} else {
				return;
			}
		}

		// Needed until static:: becomes available
		$static_class = call_user_func(array($parameters['app'] . '_report', 'instance'),array());

		// Get start / end / resolution from timeframe
		$static_class->get_timeframe($parameters);

		if($parameters['time_frame'] == self::NOW) {
			// Current values
			$data = $static_class->get_current($parameters);
		} else {
			// Historical
			$data = $static_class->get_aged($parameters);
		}

		$tmpfname = tempnam($GLOBALS['egw_info']['server']['temp_dir'],'report');
		$file = fopen($tmpfname, "w+");
		foreach($parameters as $name => $value)
		{
			if($name == 'time_frame') $value = self::$time_frame[$value];
			if(in_array($name, array('start', 'end')))
			{
				$name = lang($name);
				$value = egw_time::to($value);
			}
			elseif ($static_class->field2label[$name])
			{
				$value = $static_class->select_options[$name][$value] ? $static_class->select_options[$name][$value] : lang($value);
				$name = $static_class->field2label[$name] ? $static_class->field2label[$name] : lang($name);
			}
			elseif ($name == 'group_by')
			{
				$name = lang('Group by');
				$value = $static_class->field2label[$value];
			}
			elseif ($name == 'cf')
			{
				$cfs = config::get_customfields('tracker');
				foreach($value as $c_name => $c_field)
				{
                                        if($c_field) fwrite($file, $cfs[$c_name]['label'] . ': ' . $cfs[$c_name]['values'][$c_field]."\n");
                                }
				continue;
			}
			else
			{
				$name = lang($name);
				$value = lang($value);
			}
			fwrite($file, "$name: $value\n");
		}

		if($parameters['time_frame'] == self::NOW) {
			foreach($data as $series => $values) {
				fwrite($file, "\n$series\n");
				foreach($values as $name => $v) {
					fputcsv($file, array($name, $v), ';');
				}
			}
		} else {
			foreach($data as $series => $values) {
				$header = array_keys($values);
				array_unshift($header, false);
				fputcsv($file, $header, ';');
				break;
			}
			foreach($data as $series => $values) {
				$label = $static_class->select_options[$parameters['group_by']][$series];
				array_unshift($values, $label);
				fputcsv($file, $values, ';');
			}
		}

		fclose($file);
		$filename = $GLOBALS['egw_info']['server']['temp_dir'] .'/'. pathinfo($tmpfname, PATHINFO_FILENAME);
		header('Content-Description: File Transfer');
		header('Content-Type: text');
		header('Content-Disposition: attachment; filename=report-'.date('Y-m-d').'.csv');
		header('Content-Transfer-Encoding: binary');
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		header('Pragma: public');
		header('Content-Length: ' . filesize($filename));
		ob_clean();
		flush();
		readfile($filename);
		common::egw_exit();
	}

	/**
	 * Get start / end / resolution from selected timeframe
	 * start, end, and time_slice are added to parameters based
	 * on selected time_frame.
	 *
	 * @param parameters Array of filters used to select data
	 */
	protected static function get_timeframe(&$parameters) {
		$static_class = call_user_func(array($parameters['app'] . '_report', 'instance'),array());

		// This happens on first request, avoid an error
		if(!$parameters['time_frame']) $parameters['time_frame'] = self::NOW;

		switch($parameters['time_frame']) {
			case self::THIS_WEEK:
			case self::LAST_WEEK:
				$wday = (int) date('w'); // 0=sun, ..., 6=sat
                                switch($GLOBALS['egw_info']['user']['preferences']['calendar']['weekdaystarts'])
                                {
                                        case 'Sunday':
                                                $weekstart = strtotime('- '.$wday.' days', mktime(0,0,0,date('m'),date('d'),date('Y')));
                                                break;
                                        case 'Saturday':
                                                $weekstart = strtotime('- '.(6-$wday).' days', mktime(0,0,0,date('m'),date('d'),date('Y')));
                                                break;
                                        case 'Moday':
                                        default:
                                                $weekstart = strtotime('- '.($wday ? $wday-1 : 6).' days', mktime(0,0,0,date('m'),date('d'),date('Y')));
                                                break;
                                }
				$sweek = $parameters['time_frame'] == self::THIS_WEEK ? 0 : -1;
				$eweek = $parameters['time_frame'] == self::THIS_WEEK ? 1 : 0;
                                $parameters['start'] = $weekstart + $sweek*7*24*60*60;
                                $parameters['end'] = $weekstart + $eweek*7*24*60*60-1;
				$parameters['time_slice'] = self::DAY;
				break;
			case self::THIS_MONTH:
				$parameters['start'] = mktime(0,0,0,date('m'),1,date('Y'));
				$parameters['end'] = mktime(0,0,0,date('m'),date('t'),date('Y'));
				$parameters['time_slice'] = self::WEEK;
				break;
			case self::LAST_MONTH:
				$parameters['start'] = mktime(0,0,0,date('m')-1,1,date('Y'));
				$parameters['end'] = mktime(0,0,0,date('m'),1,date('Y'))-1;
				$parameters['time_slice'] = self::WEEK;
				break;
			case self::LAST_3_MONTHS:
				$parameters['start'] = mktime(0,0,0,date('m')-3,1,date('Y'));
				$parameters['end'] = mktime(0,0,0,date('m'),1,date('Y'))-1;
				$parameters['time_slice'] = self::WEEK;
				break;
			case self::THIS_QUARTER:
			case self::LAST_QUARTER:
				switch(date('m')) {
					case 1: case 2: case 3: $month = 1;
					case 4: case 5: case 6: $month = 4;
					case 7: case 8: case 9: $month = 7;
					case 10: case 11: case 12: $month = 10;
				}
				$parameters['start'] = mktime(0,0,0, $month, 1, date('Y'));
				$parameters['end'] = mktime(23,59,59, $month + 3, 0, date('Y'));
				$parameters['time_slice'] = self::WEEK;
				if($parameters['time_frame'] == self::THIS_QUARTER) break;
			// self::LAST_QUARTER:
				$parameters['end'] = $parameters['start'] - 1;
				$parameters['start'] = strtotime('-3 months', $parameters['start']);
				$parameters['time_slice'] = self::WEEK;
				break;
			case self::THIS_YEAR:
				$parameters['start'] = mktime(0,0,0,1,1,date('Y'));
				$parameters['end'] = mktime(0,0,0,12,31, date('Y'));
				$parameters['time_slice'] = self::MONTH;
				break;
			case self::LAST_YEAR:
				$parameters['start'] = mktime(0,0,0,1,1,date('Y')-1);
				$parameters['end'] = mktime(0,0,0,12,31, date('Y')-1);
				$parameters['time_slice'] = self::MONTH;
				break;
			case self::TWO_YEARS_AGO:
				$parameters['start'] = mktime(0,0,0,1,1,date('Y')-2);
				$parameters['end'] = mktime(0,0,0,12,31, date('Y')-2);
				$parameters['time_slice'] = self::MONTH;
				break;
			case self::THREE_YEARS_AGO:
				$parameters['start'] = mktime(0,0,0,1,1,date('Y')-3);
				$parameters['end'] = mktime(0,0,0,12,31, date('Y')-3);
				$parameters['time_slice'] = self::MONTH;
				break;
		}

		// Make sure there's a group by for historical
		if($parameters['time_frame'] != self::NOW && !$parameters['group_by']) $parameters['group_by'] = key($static_class->group_by_options);

		// Can't look into the future
		if($parameters['end'] > time() || !$parameters['end']) $parameters['end'] = time();

		// Drop the empty filters
		foreach($parameters as $key => $value) {
			// Custom fields
			if(is_array($value)) {
				foreach($value as $v_key => $v_value) {
					if(!$v_value) unset($value[$v_key]);
				}
			}
			if(!$value) unset($parameters[$key]);
		}
	}

	/**
	 * Set graph subtitle based on filter parameters
	 * @param parameters Array of filters used to select data
	 */
	protected static abstract function set_subtitle(&$graph, $parameters);

	/**
	 * Build the graph showing the historical changes
	 * Graph is a stacked bar graph, showing count changes over time
	 *
	 * @param parameters Search / filter parameters limiting results
	 * @param data Array of data used to make a stacked bar graph
	 *
	 * @return array Data for graph, in a format suitable for display.
	 */
	protected static abstract function get_history_graph(Array $parameters, Array $data);

	/**
	 * Build the graph showing the current snapshot
	 * Graph is a series of pie charts showing current breakdown, one pie per sub-array in data
	 *
	 * @param parameters Search / filter parameters limiting results
	 * @param data Array of data used to make multiple pie graphs
	 *
	 * @return array Data for graph, in a format suitable for display.
	 */
	protected static abstract function get_current_graph(Array $parameters, Array $data);

	/**
	 * Limit number of series / values to keep graph readable
	 * Works for stacked bar graph where there are several series and you need to limit the number.
	 *
	 * Default limit of 10 and 7% chosen for nice graphs, you need at least total count > 15
	 *
	 * @param data Array of series that need to be limited, each series with multiple points of x => y data
	 * @param series_limit integer limit of series to allow
	 * @param series_value (%) Series with totals less than this are candidates for merging
	 */
	protected static function limit_series(&$data, $series_limit = 10, $series_value = 0.07)
	{
		if(count($data) > $series_limit)
		{
			$total = 0;
			foreach($data as $series => $counts) {
				$total += array_sum($counts);
			}
			foreach($data as $series => $count) {
				if(array_sum($count) < $total * $series_value) {
					foreach($count as $date => $value)
					{
						// Add [] around Other in case app already has an 'Other'
						$data['['.lang('Other').']'][$date] += $value;
					}
					unset($data[$series]);
				}
			}
		}
	}

	/**
	 * Figure out the date for a given week sum
	 *
	 * http://www.phpbuilder.com/board/showpost.php?p=10777545&postcount=19
	 *
	 * @param wk_n ISO week number (1-52)
	 * @param yr Year
	 * @param format Format for returned date
	 *
	 * @return formatted date
	 */
	public static function monday_week_n($wk_n, $yr, $format = 'Y-m-d')
	{
		$ts = strtotime($yr . '-01-04 +' . ($wk_n - 1) . ' weeks');
		while (date('l', $ts) != 'Monday') {
			$ts = strtotime('-1 day', $ts);
		}
		return date($format, $ts);
	}
}
stylite_report::static_init();
?>
