<?php
/**
 * EGroupware API: VFS sharing
 *
 * @link http://www.egroupware.org
 * @license http://opensource.org/licenses/gpl-license.php GPL - GNU General Public License
 * @package api
 * @author Ralf Becker <rb@stylite.de>
 * @copyright (c) 2014 by Ralf Becker <rb@stylite.de>
 * @version $Id: class.stylite_sharing.inc.php 3469 2014-12-03 16:25:26Z ralfbecker $
 */

/**
 * EPL sharing:
 * - writable shares
 * - expiration date or password protection
 */
class stylite_sharing extends egw_sharing
{
	/**
	 * Create a new share
	 *
	 * @param string $path either path in temp_dir or vfs with optional vfs scheme
	 * @param string $mode 'link': copy file in users tmp-dir or 'share_ro' share given vfs file, if no vfs behave as 'link'
	 * @param string $name filename to use for $mode='link', default basename of $path
	 * @param string|array $recipients one or more recipient email addresses
	 * @param string $expiration =null
	 * @param string $password =null
	 * @throw egw_exception_not_found if $path not found
	 * @throw egw_excpetion_assertion_failed if user temp. directory does not exist and can not be created
	 * @return array with share data, eg. value for key 'share_token'
	 */
	public static function create($path, $mode, $name, $recipients, $expiration=null, $password=null)
	{
		$extra = array();
		if ($expiration)
		{
			$extra['share_expires'] = $expiration;
		}
		if ($password)
		{
			$extra['share_passwd'] = auth::encrypt_ldap($password, 'blowfish_crypt');
		}
		// allow filesystem path only for temp_dir --> requires link, no readonly or writable
		$temp_dir = $GLOBALS['egw_info']['server']['temp_dir'].'/';
		if (substr($path, 0, strlen($temp_dir)) == $temp_dir)
		{
			$mode = self::LINK;
		}
		$extra['share_writable'] = $mode == self::WRITABLE;

		return parent::create($path, $mode, $name, $recipients, $extra);
	}
}