/**
*	Javascript for doing a link search across all applications
*
*	@author Nathan Gray
*/

/**
*	How many applications we are searching?
*	We'll count as we go through them, and when the last response comes in,
*	hide the "Please wait..." message.
*/
var app_search_count;

/**
*	Go through each application that is set up and run the query using AJAX.
*	The response contains the code needed to display the results.
*
*	@param pattern Text to search for
*/
function search(pattern) {

	app_search_count = 0;
	var parent_node = 'exec[apps]';

	// Go through list of applications that support linking
	var apps = $j('div.app_box');
	for(var i = 0; i < apps.length; i++) {

		if(apps[i].parentNode != document.getElementById(parent_node)) {
			continue;
		}

		// Call query
		xajax_doXMLHTTP('stylite.stylite_link_search_ajax.query', 
			document.getElementById('etemplate_exec_id').value,
			apps[i].id, apps[i].getAttribute('data-app'), pattern, {'type' : apps[i].getAttribute('data-type')}
		);

		app_search_count++;

		// Update links
		if($j(apps[i]).attr('data-link')) {
			var link = document.getElementById(apps[i].id + '.more_link').firstElementChild;
			var href = $j(apps[i]).attr('data-link') + '&search=' + pattern;
			
			if(apps[i].hasAttribute('data-type') && apps[i].getAttribute('data-type') != '') {
				href += '&type=' + apps[i].getAttribute('data-type');
			}
			$j(link).wrap('<a href="' + href + '">');
		}
	}

	if(app_search_count > 0) {
		show_wait(true);
	}
}

/**
*	Show or hide a 'Please wait...' message
*/
function show_wait(display) {
	document.getElementById('exec[wait]').style.display = (display ? 'block' : 'none');
}

/**
*	Show or hide a 'More...' link
*/
function show_more(div_id, display) {
	var div = document.getElementById(div_id);
	document.getElementById(div_id + '.more_link').style.display = 
		(display && div.hasAttribute('data-link') && div.getAttribute('data-link') != '' ? 'block' : 'none');
}
