/**
 * Graphing using jqPlot
 *
 * @author Nathan Gray
 * $Id: jqplot.js 2720 2013-10-24 20:26:03Z nathangray $
 */

egw_LAB.wait(function() {

	var et2 = null;
	var ready = function(event, _et2) {

		et2 = _et2;

		// jqPlot uses passed IDs, and the [] cause issues
		$j(document.getElementById('exec[chart]')).attr('id', 'chart');
		$j(document.getElementById('exec[legend]')).attr('id', 'legend');

		get_graph_data();
		// Add change handlers to all parameters
		$j(et2.widgetContainer.getWidgetById('parameters').getDOMNode())
			.on('change',':input', function() {
			// Clear existing report name
			et2.widgetContainer.getWidgetById('report_name').set_value('');
			get_graph_data();
		});
		
		// Show / hide group by based on time frame
		var timeframe = et2.widgetContainer.getWidgetById('time_frame');
		var group_by = et2.widgetContainer.getWidgetById('group_by');
		if(timeframe.get_value() == 'NOW') group_by.set_disabled(true);
		timeframe.change = function() {
			if(this.get_value() == 'NOW') {
				group_by.set_disabled(true);
			} else {
				group_by.set_disabled(false);
			}
			get_graph_data();
		};
	};

/**
 * Request data to generate graph(s)
 *
 * @param form Current form object, filters & parameters are sent from here
 */
var get_graph_data = function() {
	// Display loading icon
	$j('.ajax-loader').show();

	var app = egw_getAppName();
	var params = {};
	params = et2.getValues(et2.widgetContainer.getWidgetById('parameters'));
	params.parameters.app = app;
	var menuaction = 'stylite.'+app+'_report.ajax_chart'+'./etemplate/process_exec';
	var req = egw.json('stylite.'+app+'_report.ajax_chart',[params],draw_graph)
		.sendRequest(true);
};

/**
 * Callback to draw graphs with given data
 *
 * @param data Data for graphs
 */
var draw_graph = function(data) {
	// Hide loading icon
	$j('.ajax-loader').hide();

	if(typeof data == 'string') {
		// error
		console.error(data);
		return;
	} else if (typeof data == 'object') {
		var div = chart_div = $j('div.chart_container').attr('id');
		var legend_div = 'legend';
		var _div = _chart_div = $j(document.getElementById(div));

		_div.empty();
		$j(document.getElementById(legend_div)).empty();

		// Remove title(s), they may have changed and we don't want duplicates
		$j('.chart_title',document.getElementById(chart_div).parentNode).remove();
		for(var i = 0; i < data.length; i++) {
			// Create graph(s)
			chart_div = div + '_'+i+'_chart';
			_chart_div = $j(document.getElementById(chart_div));

			if(!_chart_div.length && document.getElementById(div)) {
				// Create new div to hold chart
				_div.append('<div class="' + data[i].type + '"><div class="chart_title">'+
				data[i].title+
				(data[i].subtitle || data[i].subsubtitle ?
					'<div class="subtitle">'+data[i].subtitle+'</div><div class="subsubtitle">'+data[i].subsubtitle+'</div>' :
					''
				)+
					'</div><div class="chart" id="'+chart_div+'"/></div>');
			}
			try {
				if(data[i].type && data[i].type == "Pie") {
					draw_pie(chart_div, data[i]);
				} else {
					draw_bars(chart_div, data[i]);
				}
			} catch(e){
				console.error(e);
				console.log("Data: ", data[i]);
				continue;
			}
		}
	}
};

/**
 * Draw a bar chart
 */
var draw_bars = function (target, data) {
	// Map out series.  Can handle multiple series
	var series = new Array();
	var series_options = new Array();
	for(var n = 0; n < data.series.length; n++) {
		series_options[n] = {
			label: data.series[n].legend
		};
		series[n] = data.series[n].values;
		var series_data = data.series[n];
		for(var i = 0; i < series[n].length; i++) {
			// Create tooltip
			$j('#'+target).parent().append("<div class='tooltip' id='"+target+"_"+n+"_"+i+"'><p>"+data.series[n].tooltip[i]+"</p></div>");
		}
	}
	var chart = jQuery.jqplot(target, series, {
		// Use specific divs for title
		//title:	data.title, 
		stackSeries: true,
		seriesDefaults:{
			renderer:jQuery.jqplot.BarRenderer,
				rendererOptions: {
				// Put a 30 pixel margin between bars.
				barMargin: 30
			}
		},
		series: series_options,
		axes: {
			xaxis: {
				renderer: jQuery.jqplot.CategoryAxisRenderer,
				label: data.x_label,
				ticks: data.x_labels
			},
			yaxis: {
				label: data.y_label,
				labelRenderer: jQuery.jqplot.CanvasAxisLabelRenderer,
				// Don't pad out the bottom of the data range.  By default,
				// axes scaled as if data extended 10% above and below the
				// actual range to prevent data points right on grid boundaries.
				// Don't want to do that here.
				padMin: 0
			}
		},
		legend: {
			show: true,
			location: 'ne',
			placement: 'outsideGrid'
		}  
	});

	// Bind events
	chart.target.bind('jqplotDataHighlight', chart, bar_highlight );
	chart.target.bind('jqplotDataUnhighlight', chart, bar_unhighlight );
};

/**
 * Draw a pie chart
 */
var draw_pie = function (target, data) {

	// Map out series.  Can handle multiple series
	var series = new Array();
	var series_count = (data.series.length > 0 && typeof data.series[0] == 'object') ? 1 : data.series.length;
	for(var n = 0; n < series_count; n++) {
		series[n] = new Array();
		var series_data = series_count == 1 ? data.series : data.series[n];
		for(var i = 0; i < series_data.length; i++) {
			series[n][i] = [series_data[i].legend, series_data[i].value];

			// Create tooltip
			$j('#'+target).parent().append("<div class='tooltip' id='"+target+"_"+n+"_"+i+"'><p>"+series_data[i].tooltip+"</p></div>");
		}
	}
	var chart = jQuery.jqplot(target, series, {
		// Use specific divs for title
		//title:	data.title,
		seriesDefaults: {
			// Make this a pie chart.
			renderer: jQuery.jqplot.PieRenderer, 
			rendererOptions: {
				// Hide data labels on the pie slices.
				// By default, labels show the percentage of the slice.
				showDataLabels: false
			}
		},
		legend: { show:true, location: 'e' }
	});

	// Bind events
	chart.target.bind('jqplotDataHighlight', chart, pie_highlight );
	chart.target.bind('jqplotDataUnhighlight', chart, pie_unhighlight );
};

/**
 * Pull out the current slice, and show the tooltip
 *
 * @param e Event
 * @param sidx Series index - currently highlighted series
 * @param pidx Series data index - currently highlighted slice
 * @param data Data for hightlighted slice
 */
var pie_highlight = function (e, sidx, pidx, data) {
	var plot = e.data;
	var s = plot.series[sidx];

	s._highlightedPoint = pidx;
	plot.plugins.pieRenderer.highlightedSeriesIndex = sidx;

	// Hide tooltip(s)
	$j('.tooltip', this.parent).stop(true,true).hide();

	var position = plot.target.position();
	var center = s._center;
	var slice = plot.series[sidx]._sliceAngles[pidx];
	var radius = s._diameter / 2; // s._radius gets changed in animation

	// Display current tooltip centered on slice at outside edge
	var tooltip_id = plot.targetId + '_' + sidx + '_' + pidx;
	var position = {x: position.left + center[0], y:position.top + center[1]};
	var slice_center = (slice[1] - slice[0])/2 + slice[0];
	var cos = Math.cos(slice_center);
	var sin = Math.sin(slice_center);

	position.x = Math.max(0,position.x + cos*(radius + (cos < 0 ? $j(tooltip_id).width():0) + 5)); // max() to keep it on the page
	position.y = position.y + sin*(radius + $j(tooltip_id).height() + 5);
	$j(tooltip_id).css({left: position.x, top: position.y}).show();
	

	// jQuery animate on canvas from http://acko.net/blog/abusing-jquery-animate-for-fun-and-profit-and-bacon
	if(plot.animate) $j(plot.animate).stop(true,true);
	var vars = plot.animate ? plot.animate : $j.extend($j('<div>')[0], {
		// Needed for drawing
		series: s,
		sIndex: sidx,
		pIndex: pidx,

		// Animate these
		radius: s._radius,
		margin: s.sliceMargin,

		// Keep for after
		oldRadius: s._radius,
		oldMargin: s.sliceMargin,

		// Used for animating
		customAnimate: true,
		updated: true
	});
	plot.animate = vars;
	
	var fx_step_default = $j.fx.step._default;
	$j.fx.step._default = function (fx) {
		if (!fx.elem.customAnimate) return $_fx_step_default(fx);
		fx.elem[fx.prop] = fx.now;
		fx.elem.updated = true;
	};

	var interval_id = setInterval(function() {
		if(!vars.updated) return;
		vars.updated = false;

		// Change values
		vars.series._radius = vars.radius;
		vars.series.sliceMargin = vars.margin;

		// Re-draw
		var canvas = plot.plugins.pieRenderer.highlightCanvas;
		canvas._ctx.clearRect(0,0,canvas._ctx.canvas.width, canvas._ctx.canvas.height);
		vars.series.renderer.drawSlice.call(
			vars.series, 
			canvas._ctx, 
			vars.series._sliceAngles[vars.pIndex][0],
			vars.series._sliceAngles[vars.pIndex][1], 
			vars.series.highlightColorGenerator.get(vars.pIndex), 
			false
		);
	},30);

	$j(vars).animate({ radius: vars.radius+10 }, { duration: 300, complete: function() {
		// Reset
		vars.series.sliceMargin = vars.oldMargin;
		vars.series._radius = vars.series._diameter / 2;  // Use diameter, because it never changed
		clearInterval(interval_id);
		$j(plot.animate).stop();
		plot.animate = null;
	} });
};

/**
 * Hide tooltip.  Hiding large slice handled by jqPlot.
 */
var pie_unhighlight = function(e) {
	// Hide tooltip(s)
	$j('.tooltip', this.parent).stop(true,true).hide();
};


/**
 * Show the tooltip
 *
 * @param e Event
 * @param sidx Series index - currently highlighted series
 * @param pidx Series data index - currently highlighted bar (segment)
 * @param data Data for hightlighted segment
 */
var bar_highlight = function (e, sidx, pidx, data) {
	var plot = e.data;
	// Hide tooltip(s)
	$j('.tooltip', this.parent).stop(true,true).hide();

	var position = plot.target.position();
	// Get top of bar
	var bar = plot.series[sidx].gridData[pidx];

	// Display current tooltip
	var tooltip_id = plot.targetId + '_' + sidx + '_' + pidx;
	$j(tooltip_id).css({left: position.left + bar[0] + plot.series[sidx].barWidth, top: position.top + bar[1]}).show();
};

/**
 * Hide tooltip.  
 */
var bar_unhighlight =function (e) {
	// Hide tooltip(s)
	$j('.tooltip', this.parent).stop(true,true).hide();
};

$j('.et2_container').on('load', ready);
});
