<?php
/**
 * Stylite: Nagios checks for logged in users
 *
 * @link http://www.stylite.de
 * @package stylite
 * @author Ralf Becker <rb-at-stylite.de>
 * @copyright (c) 2012 by Ralf Becker <rb-at-stylite.de>
 * @version $Id: nagios.php 2108 2012-05-19 18:11:49Z ralfbecker $
 */

$GLOBALS['egw_info'] = array(
	'flags' => array(
		'currentapp' => 'login',
	)
);

include('../header.inc.php');

$valid_ips = $GLOBALS['egw_info']['server']['nagios_session_ips'];

if (!isset($valid_ips) || !preg_match($valid_ips, $_SERVER['REMOTE_ADDR']) || !$_GET ||
	($error = parse_parameter($filter, $dla_critical, $count_op, $count_warning, $count_critical, $debug, $msg)))
{
?>
<!DOCTYPE unspecified PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
	<title>EGroupware Nagios session-check</title>
</head>
<body>
	<h1>EGroupware Nagios session-check</h1>
	<p>This service can be used to check if specified users are logged into EGroupware.</p>
<?php
	if (!isset($valid_ips))
	{
		echo "<p>The service is <b>NOT</b> enabled!</p>\n";
	}
	elseif(!preg_match($valid_ips, $_SERVER['REMOTE_ADDR']))
	{
		echo "<p>Your IP ".htmlspecialchars($_SERVER['REMOTE_ADDR'])." is <b>NOT</b> enabled for this service!</p>\n";
	}
	elseif(isset($error))
	{
		echo "<p><b>ERROR: $error</b></p>\n";
	}
?>
	<p>To enable the service for your current IP <b><?php echo $_SERVER['REMOTE_ADDR']; ?></b> you have to either:
	<ul>
		<li>add the following line to your header.inc.php to enable it for all EGroupware instances:<br />
			<b>$GLOBALS['egw_info']['server']['nagios_session_ips'] = '/^<?php echo preg_quote($_SERVER['REMOTE_ADDR'],'/'); ?>$/';</b></li>
		<li>add the following to an EGroupware database to enable it for that EGroupware instance:<br />
			<b>INSERT INTO egw_config (config_app,config_name,config_value) VALUES ('phpgwapi','nagios_session_ips','/^<?php echo preg_quote($_SERVER['REMOTE_ADDR'],'/'); ?>$/')</b></li>
		<li>change line 20 of this script to:<br />
			<b>$valid_ips = '/^<?php echo preg_quote($_SERVER['REMOTE_ADDR'],'/'); ?>$/';</b></li>
	</ul>
	<h3>You can use the following URL parameters to configure the check in Nagios:</h3>
	<ul>
		<li><b>account_id</b>=&lt;nummerical account id&gt; <i>eg. account_id=5</i></li>
		<li><b>account_lid</b>=&lt;account name&gt; <i>eg. account_lid=ralf</i></li>
		<li><b>ip</b>=&lt;ip address of session user&gt; <i>eg. ip=192.168.1.1 or ip=192.168.1.*</i></li>
		<li><b>action</b>=&lt;last action as in session list&gt; <i>eg. action=/webdav.php/*</i><br />
			you can use wildcards * or ? for ip or action, a <b>!</b> as first char negates the expression</li>
		<li><b>idle</b>(&lt;|&lt;=|=|&gt;=|&gt;)&lt;seconds for warning&gt;[:&lt;seconds for critical&gt;]<br />
			no session is treated like an expired idle time!</li>
		<li><b>count</b>(&lt;|&lt;=|=|&gt;=|&gt;)&lt;count for warning&gt;[:&lt;count for critical&gt;]</li>
		<li><b>msg</b>=&lt;custom message for check to display in nagios&gt; default "N sessions for (&lt;criteria&gt;)"<br />
			"NOT" get's removed from this message for serverity "OK"</li>
		<li><b>debug</b>=1 shows a session list for the given filter</li>
	</ul>
	<p>At least one parameter <b>dla</b> or <b>count</b> is required!</p>
	<h3>The check will return the following, which can be easily evaluated in Nagios:</h3>
	<ul>
		<li><b>OK:</b> [&lt;custom message (with NOT removed)&gt;: ]N sessions for (&lt;criteria&gt;)</li>
		<li><b>WARNING:</b> [&lt;custom message&gt;: ]0 sessions for (&lt;criteria&gt;)</li>
		<li><b>ERROR:</b> [&lt;custom message&gt;: ]0 sessions for (&lt;criteria&gt;)</li>
	</ul>
	<h3>Examples:</h3>
	<ul>
<?php
	$nagios_url = ($_SERVER['HTTPS'] ? 'https://' : 'http://').$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'];
	foreach(array(
		'account_lid=ralf&idle>300:600&msg=Ralf+NOT+active&action=!/*dav.php*' => 'Gives WARNING/ERROR if user "ralf" is idle for more then 5/10 minutes excluding WebDAV and GroupDAV sessions',
		'account_lid=admin&count>0&msg=Admin is logged in' => 'Gives WARNING if user "admin" is logged into EGroupware',
	) as $query => $description)
	{
		echo '<li><a href="'.htmlspecialchars($nagios_url.'?'.$query).'" target="_blank">'.htmlspecialchars($nagios_url.'?'.$query)."</a><br />\n$description</li>\n";
	}
?>
	</ul>
</body>
</html>
<?php
	common::egw_exit();
}
// correctly configured and parsed nagios check
try {
	header("Content-Type: text/plain; charset=utf-8");
	$serverity = check_sessions($filter, $count_op, $count_warning, $count_critical, $count, $debug);

	// check if we might have a critical dla
	if (isset($dla_critical) && $serverity == 'WARNING')
	{
		$critical_filter = preg_replace('/session_dla\)(<=|<|=|>|=>)([0-9]+)/', 'session_dla)${1}'.$dla_critical, $filter);
		$count_backup = $count;
		if (($serverity = check_sessions($critical_filter, $count_op, 1, 1, $count)) == 'OK')
		{
			$serverity = 'WARNING';
			$count = $count_backup;
		}
		else
		{
			$filer = $critical_filter;
		}
	}

	if (isset($msg) && $serverity == 'OK')	// remove evtl. negation from message
	{
		$msg = preg_replace('/(NOT|NICHT) ?/i', '', $msg);
	}
	echo $serverity.': '.(isset($msg) ? $msg.': ' : '').$count.' session(s) for '.substr(array2string($filter),5)."\n";
	if ($debug)
	{
		$sessions = egw_session::session_list(0, null, null, true, $critical_filter ? $critical_filter : $filter);
		foreach($sessions as &$session)
		{
			unset($session['session_php']);	// kill session-id for security reasons
			$session['session_dla'] = date('Y-m-d H:i:s', $session['session_dla']).' --> idle since '.(time()-$session['session_dla']);
			$session['li'] = date('Y-m-d H:i:s', $session['li']);
			if ($session['notification_heartbeat']) $session['notification_heartbeat'] = date('Y-m-d H:i:s', $session['notification_heartbeat']);
		}
		if ($sessions) print_r($sessions);
	}
}
catch (Exception $e) {
	echo 'ERROR: '.$e->getMessage();
}

/**
 * Get number of sessions for given filter and check severity
 *
 * @param array $filter for egw_session::session_count
 * @param string $count_op <, <=, =, =>, >
 * @param int $count_warning count for severity "WARNING"
 * @param int $count_critical=null optional count for severity "ERROR"
 * @param int &$count=null on return number of sessions
 * @return string serverity "ERROR", "WARNING" or "OK"
 */
function check_sessions(array $filter, $count_op, $count_warning, $count_critical, &$count=null)
{
	$count = (int)egw_session::session_count($filter);

	switch($count_op)
	{
		case '<=':
			$serverity = (isset($count_critical) && $count <= $count_critical) ? 'ERROR' :
				($count <= $count_warning ? 'WARNING' : 'OK');
			break;
		case '<':
			$serverity = (isset($count_critical) && $count < $count_critical) ? 'ERROR' :
				($count < $count_warning ? 'WARNING' : 'OK');
			break;
		case '=':
			$serverity = (isset($count_critical) && $count == $count_critical) ? 'ERROR' :
				($count == $count_warning ? 'WARNING' : 'OK');
			break;
		case '>=':
			$serverity = (isset($count_critical) && $count >= $count_critical) ? 'ERROR' :
				($count >= $count_warning ? 'WARNING' : 'OK');
			break;
		case '>':
			$serverity = (isset($count_critical) && $count > $count_critical) ? 'ERROR' :
				($count > $count_warning ? 'WARNING' : 'OK');
			break;
		default:
			$serverity = "ERROR: Invalid operator '$count_op'";
			break;
	}
	return $serverity;
}

/**
 * Parse URL parameters
 *
 * @param array &$filter for egw_session::session_count
 * @param int &$dla_critical optional nount for severity "ERROR" for dla
 * @param string &$count_op <, <=, =, =>, >
 * @param int &$count_warning count for severity "WARNING"
 * @param int &$count_critical optional count for severity "ERROR"
 * @param boolean &$debug
 * @param string &$msg optional message to display in Nagios
 * @return string with error-message or null for ok
 */
function parse_parameter(&$filter, &$dla_critical, &$count_op, &$count_warning, &$count_critical, &$debug, &$msg)
{
	$filter = array();
	$dla_critical = $count_critical = $debug = null;

	foreach($_GET as $name => $value)
	{
		switch($name)
		{
			case 'account_lid':
				if ((!$value = $GLOBALS['egw']->accounts->name2id($value)))
				{
					return $error = "Account <b>".htmlspecialchars($_GET['account_lid'])."</b> NOT found!";
				}
				// fall through
			case 'account_id':
				$filter['account_id'] = (int)$value;
				break;

			case 'ip':
			case 'action':
				$op = $value[0] === '!' ? ' NOT LIKE ' : ' LIKE ';
				if ($value[0] === '!') $value = substr($value, 1);
				$filter[] = 'session_'.$name.$op.$GLOBALS['egw']->db->quote(strtr($value,array('*'=>'%','?'=>'_','%'=>'\\%','_'=>'\\_')));
				break;

			case 'msg':
				$msg = $value;
				break;

			case 'debug':
				$debug = (boolean)$value;
				break;

			default:
				$value = $name.($value !== '' ? '='.$value : '');
				if (!preg_match('/^(idle|count)(<=|<|=|=>|>)([0-9]+)(:[0-9]+)?$/',$value,$matches))
				{
					return $error = 'Invalid syntax: '.htmlspecialchars($value);
				}
				if ($matches[1] == 'idle')
				{
					$filter[] = 'NOT ('.time().'-session_dla)'.$matches[2].(int)$matches[3];
					$dla_critical = (string)$matches[4] !== '' ? (int)substr($matches[4],1) : null;
					if (!isset($count_op))
					{
						$count_op = '<';
						$count_warning = 1;
					}
				}
				else
				{
					$count_op = $matches[2];
					$count_warning = (int)$matches[3];
					$count_critical = (string)$matches[4] !== '' ? (int)substr($matches[4],1) : null;
				}
				break;
		}
	}
	if (!isset($count_op))
	{
		return "At least one parameter <b>dla</b> or <b>count</b> is required!";
	}
	return null;
}