<?php
/**
 * Stylite: VFS - stream wrapper for linked files
 *
 * @link http://www.stylite.de
 * @package stylite
 * @subpackage links
 * @author Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @copyright (c) 2008-15 by Ralf Becker <RalfBecker-AT-outdoor-training.de>
 * @version $Id: PluginIface.php 3557 2015-01-26 09:24:34Z ralfbecker $
 */

namespace EGroupware\Stylite\Vfs\Links;

/**
 * Interface for application plugins rendering the application directory
 * to add virtual directories for easier browsing of the application entries.
 *
 * Both public methods (stat and appdir) work by populating stylite_links_stream_wrapper's stat cache
 * using the following methods:
 *
 * - void StreamWrapper::addStatCache(string $path,array|boolean $data);
 * - void StreamWrapper::addStatCacheDir(string $path,string $mime=self::DIR_MIME_TYPE);
 * - void StreamWrapper::addStatCacheLink(string $path,string $link);
 *
 * In general each virtual directory should have an english name implementing the functionality
 * (gets added with addStatCacheDir) and a translated name added as link to english one (addStatCacheLink).
 * As the links get resolved by the vfs layer, the plugin then has only to deal with the english name.
 */
interface PluginIface
{
	/**
	 * Stat something in the application directory
	 *
	 * An implementation can eg. call the appdir method of one level higher, to populate the stat cache
	 * with everything allowed on that level --> StreamWrapper::url_stat then looks up the cache
	 *
	 * With late static binding (php5.3+) this could be done by adding the code here and
	 * converting the interface into an abstract base class.
	 *
	 * @param string $id filename in the application directory
	 */
	static public function stat($id);

	/**
	 * Return directory entries for application and populate the stat cache
	 *
	 * @param string $id filename in the application directory
	 * @return array with entries to return to the user
	 */
	static public function appdir($id);
}
